"""The `mailjet_rest` package provides a Python client for interacting with the Mailjet API.

This package includes the main `Client` class for handling API requests, along with
utility functions for version management. The package exposes a consistent interface
for Mailjet API operations.

Attributes:
    __version__ (str): The current version of the `mailjet_rest` package.
    __all__ (list): Specifies the public API of the package, including `Client`
        for API interactions and `get_version` for retrieving version information.

Modules:
    - client: Defines the main API client.
    - utils.version: Provides version management functionality.
"""

from mailjet_rest.client import Client
from mailjet_rest.utils.version import get_version


__version__: str = get_version()

__all__ = ["Client", "get_version"]



import json
import os

from mailjet_rest import Client


mailjet30 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
)

mailjet31 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
    version="v3.1",
)


def event_based_vs_message_based_stats_timing():
    """GET https://api.mailjet.com/v3/REST/statcounters"""
    filters = {
        "SourceId": "$Campaign_ID",
        "CounterSource": "Campaign",
        "CounterTiming": "Message",
        "CounterResolution": "Lifetime",
    }
    return mailjet30.statcounters.get(filters=filters)


def view_the_spread_of_events_over_time():
    """GET https://api.mailjet.com/v3/REST/statcounters"""
    filters = {
        "SourceId": "$Campaign_ID",
        "CounterSource": "Campaign",
        "CounterTiming": "Event",
        "CounterResolution": "Day",
        "FromTS": "123",
        "ToTS": "456",
    }
    return mailjet30.statcounters.get(filters=filters)


def statistics_for_specific_recipient():
    """GET https://api.mailjet.com/v3/REST/contactstatistics"""
    return mailjet30.contactstatistics.get()


def stats_for_clicked_links():
    """GET https://api.mailjet.com/v3/REST/statistics/link-click"""
    filters = {"CampaignId": "$Campaign_ID"}
    return mailjet30.statistics_linkClick.get(filters=filters)


def mailbox_provider_statistics():
    """GET https://api.mailjet.com/v3/REST/statistics/recipient-esp"""
    filters = {"CampaignId": "$Campaign_ID"}
    return mailjet30.statistics_recipientEsp.get(filters=filters)


def geographical_statistics():
    """GET https://api.mailjet.com/v3/REST/geostatistics"""
    return mailjet30.geostatistics.get()


if __name__ == "__main__":
    result = geographical_statistics()
    print(result.status_code)
    try:
        print(json.dumps(result.json(), indent=4))
    except json.decoder.JSONDecodeError:
        print(result.text)



import json
import os

from mailjet_rest import Client


mailjet30 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
)

mailjet31 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
    version="v3.1",
)


def create_a_template():
    """POST https://api.mailjet.com/v3/REST/template"""
    data = {
        "Author": "John Doe",
        "Categories": "array",
        "Copyright": "Mailjet",
        "Description": "Used to send out promo codes.",
        "EditMode": "1",
        "IsStarred": "false",
        "IsTextPartGenerationEnabled": "true",
        "Locale": "en_US",
        "Name": "Promo Codes",
        "OwnerType": "user",
        "Presets": "string",
        "Purposes": "array",
    }
    return mailjet30.template.create(data=data)


def create_a_template_detailcontent():
    """POST https://api.mailjet.com/v3/REST/template/$template_ID/detailcontent"""
    _id = "$template_ID"
    data = {
        "Headers": "",
        "Html-part": "<h3>Dear passenger, welcome to Mailjet!</h3><br />May the delivery force be with you!",
        "MJMLContent": "",
        "Text-part": "Dear passenger, welcome to Mailjet! May the delivery force be with you!",
    }
    return mailjet30.template_detailcontent.create(id=_id, data=data)


def use_templates_with_send_api():
    """POST https://api.mailjet.com/v3.1/send"""
    data = {
        "Messages": [
            {
                "From": {"Email": "pilot@mailjet.com", "Name": "Mailjet Pilot"},
                "To": [{"Email": "passenger1@mailjet.com", "Name": "passenger 1"}],
                "TemplateID": 1,
                "TemplateLanguage": True,
                "Subject": "Your email flight plan!",
            },
        ],
    }
    return mailjet31.send.create(data=data)


if __name__ == "__main__":
    result = create_a_template()
    print(result.status_code)
    try:
        print(json.dumps(result.json(), indent=4))
    except json.decoder.JSONDecodeError:
        print(result.text)



import json
import os
from pathlib import Path

from mailjet_rest import Client


mailjet30 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
)

mailjet31 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
    version="v3.1",
)


def create_a_contact():
    """POST https://api.mailjet.com/v3/REST/contact"""
    data = {
        "IsExcludedFromCampaigns": "true",
        "Name": "New Contact",
        "Email": "passenger@mailjet.com",
    }
    return mailjet30.contact.create(data=data)


def create_contact_metadata():
    """POST https://api.mailjet.com/v3/REST/contactmetadata"""
    data = {"Datatype": "str", "Name": "first_name", "NameSpace": "static"}
    return mailjet30.contactmetadata.create(data=data)


def edit_contact_data():
    """PUT https://api.mailjet.com/v3/REST/contactdata/$contact_ID"""
    _id = "*********"  # Put real ID to make it work.
    data = {"Data": [{"Name": "first_name", "Value": "John"}]}
    return mailjet30.contactdata.update(id=_id, data=data)


def manage_contact_properties():
    """POST https://api.mailjet.com/v3/REST/contactmetadata"""
    _id = "$contact_ID"
    data = {"Data": [{"Name": "first_name", "Value": "John"}]}
    return mailjet30.contactdata.update(id=_id, data=data)


def create_a_contact_list():
    """POST https://api.mailjet.com/v3/REST/contactslist"""
    data = {"Name": "my_contactslist"}
    return mailjet30.contactslist.create(data=data)


def add_a_contact_to_a_contact_list():
    """POST https://api.mailjet.com/v3/REST/listrecipient"""
    data = {
        "IsUnsubscribed": "true",
        "ContactID": "987654321",
        "ContactAlt": "passenger@mailjet.com",
        "ListID": "123456",
        "ListAlt": "abcdef123",
    }
    return mailjet30.listrecipient.create(data=data)


def manage_the_subscription_status_of_an_existing_contact():
    """POST https://api.mailjet.com/v3/REST/contact/$contact_ID/managecontactslists"""
    _id = "$contact_ID"
    data = {
        "ContactsLists": [
            {"Action": "addforce", "ListID": "987654321"},
            {"Action": "addnoforce", "ListID": "987654321"},
            {"Action": "remove", "ListID": "987654321"},
            {"Action": "unsub", "ListID": "987654321"},
        ],
    }
    return mailjet30.contact_managecontactslists.create(id=_id, data=data)


def manage_multiple_contacts_in_a_list():
    """POST https://api.mailjet.com/v3/REST/contactslist/$list_ID/managemanycontacts"""
    _id = "$list_ID"
    data = {
        "Action": "addnoforce",
        "Contacts": [
            {
                "Email": "passenger@mailjet.com",
                "IsExcludedFromCampaigns": "false",
                "Name": "Passenger 1",
                "Properties": "object",
            },
        ],
    }
    return mailjet30.contactslist_managemanycontacts.create(id=_id, data=data)


def monitor_the_upload_job():
    """GET https://api.mailjet.com/v3/REST/contactslist/$list_ID/managemanycontacts"""
    _id = "$list_ID"
    return mailjet30.contactslist_managemanycontacts.get(id=_id)


def manage_multiple_contacts_across_multiple_lists():
    """POST https://api.mailjet.com/v3/REST/contact/managemanycontacts"""
    data = {
        "Contacts": [
            {
                "Email": "passenger@mailjet.com",
                "IsExcludedFromCampaigns": "false",
                "Name": "Passenger 1",
                "Properties": "object",
            },
        ],
        "ContactsLists": [
            {"Action": "addforce", "ListID": "987654321"},
            {"Action": "addnoforce", "ListID": "987654321"},
            {"Action": "remove", "ListID": "987654321"},
            {"Action": "unsub", "ListID": "987654321"},
        ],
    }
    return mailjet30.contact_managemanycontacts.create(data=data)


def upload_the_csv():
    """POST https://api.mailjet.com/v3/DATA/contactslist
    /$ID_CONTACTLIST/CSVData/text:plain"""
    return mailjet30.contactslist_csvdata.create(
        id="$ID_CONTACTLIST",
        data=Path("./data.csv").read_text(encoding="utf-8"),
    )


def import_csv_content_to_a_list():
    """POST https://api.mailjet.com/v3/REST/csvimport"""
    data = {
        "ErrThreshold": "1",
        "ImportOptions": "",
        "Method": "addnoforce",
        "ContactsListID": "123456",
        "DataID": "98765432123456789",
    }
    return mailjet30.csvimport.create(data=data)


def using_csv_with_atetime_contact_data():
    """POST https://api.mailjet.com/v3/REST/csvimport"""
    # fmt: off
    data = {
        "ContactsListID": "$ID_CONTACTLIST",
        "DataID": "$ID_DATA",
        "Method": "addnoforce",
        "ImportOptions": "{\"DateTimeFormat\": \"yyyy/mm/dd\","
                         "\"TimezoneOffset\": 2,\"FieldNames\": "
                         "[\"email\", \"birthday\"]} ",
    }
    # fmt: on
    return mailjet30.csvimport.create(data=data)


def monitor_the_import_progress():
    """GET https://api.mailjet.com/v3/REST/csvimport/$importjob_ID"""
    _id = "$importjob_ID"
    return mailjet30.csvimport.get(id=_id)


def error_handling():
    """https://api.mailjet.com/v3/DATA/BatchJob/$job_id/CSVError/text:csv"""
    """Not available in Python, please refer to Curl"""


def single_contact_exclusion():
    """PUT https://api.mailjet.com/v3/REST/contact/$ID_OR_EMAIL"""
    _id = "$ID_OR_EMAIL"
    data = {"IsExcludedFromCampaigns": "true"}
    return mailjet30.contact.update(id=_id, data=data)


def using_contact_managemanycontacts():
    """POST https://api.mailjet.com/v3/REST/contact/managemanycontacts"""
    data = {
        "Contacts": [
            {
                "Email": "jimsmith@example.com",
                "Name": "Jim",
                "IsExcludedFromCampaigns": "true",
                "Properties": {"Property1": "value", "Property2": "value2"},
            },
            {
                "Email": "janetdoe@example.com",
                "Name": "Janet",
                "IsExcludedFromCampaigns": "true",
                "Properties": {"Property1": "value", "Property2": "value2"},
            },
        ],
    }
    return mailjet30.contact_managemanycontacts.create(data=data)


def using_csvimport():
    """POST https://api.mailjet.com/v3/REST/csvimport"""
    data = {"DataID": "$ID_DATA", "Method": "excludemarketing"}
    return mailjet30.csvimport.create(data=data)


def retrieve_a_contact():
    """GET https://api.mailjet.com/v3/REST/contact/$CONTACT_EMAIL"""
    _id = "$CONTACT_EMAIL"
    return mailjet30.contact.get(id=_id)


def delete_the_contact():
    """DELETE https://api.mailjet.com/v4/contacts/{contact_ID}"""


if __name__ == "__main__":
    result = edit_contact_data()
    print(result.status_code)
    try:
        print(json.dumps(result.json(), indent=4))
    except json.decoder.JSONDecodeError:
        print(result.text)



import json
import os

from mailjet_rest import Client


mailjet30 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
)

mailjet31 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
    version="v3.1",
)

if __name__ == "__main__":
    from samples.contacts_sample import edit_contact_data

    result = edit_contact_data()
    print(result.status_code)
    try:
        print(json.dumps(result.json(), indent=4))
    except json.decoder.JSONDecodeError:
        print(result.text)



"""Versioning utilities for the Mailjet REST API client.

This module defines the current version of the Mailjet client and provides
a helper function, `get_version`, to retrieve the version as a formatted string.

Attributes:
    VERSION (tuple[int, int, int]): A tuple representing the major, minor, and patch
    version of the package.

Functions:
    get_version: Returns the version as a string in the format "major.minor.patch".
"""

from __future__ import annotations

import re

from mailjet_rest._version import __version__ as package_version


def clean_version(version_str: str) -> tuple[int, ...]:
    """Clean package version string into 3 item tuple.

    Parameters:
    version_str (str): A string of the package version.

    Returns:
    tuple: A tuple representing the version of the package.
    """
    if not version_str:
        return 0, 0, 0
    # Extract just the X.Y.Z part using regex
    match = re.match(r"^(\d+\.\d+\.\d+)", version_str)
    if match:
        version_part = match.group(1)
        return tuple(map(int, version_part.split(".")))

    return 0, 0, 0  # type: ignore[unreachable]


# VERSION is a tuple of integers (1, 3, 2).
VERSION: tuple[int, ...] = clean_version(package_version)


def get_version(version: tuple[int, ...] | None = None) -> str:
    """Calculate package version based on a 3 item tuple.

    In addition, verify that the tuple contains 3 items.

    Parameters:
    version (tuple[int, ...], optional): A tuple representing the version of the package.
        If not provided, the function will use the VERSION constant.
        Default is None.

    Returns:
    str: The version as a string in the format "major.minor.patch".

    Raises:
    ValueError: If the provided tuple does not contain exactly 3 items.
    """
    if version is None:
        version = VERSION
    if len(version) != 3:
        msg = "The tuple 'version' must contain 3 items"
        raise ValueError(msg)
    return "{}.{}.{}".format(*(x for x in version))



"""The `mailjet_rest.utils` package provides utility functions for interacting with the package versionI.

This package includes a module for managing the package version.

Modules:
    - version: Manages the package versioning.
"""



from __future__ import annotations
from functools import partial

import glob
import json
import os
import re
from datetime import datetime
from pathlib import Path
from typing import Any

import pytest
from _pytest.logging import LogCaptureFixture

from mailjet_rest.utils.version import get_version
from mailjet_rest import Client
from mailjet_rest.client import prepare_url, parse_response, logging_handler, Config


def debug_entries() -> tuple[str, str, str, str, str, str, str]:
    """Provide a simple tuples with debug entries for testing purposes.

    Parameters:
    None

    Returns:
    tuple: A tuple containing seven debug entries
    """
    entries = (
        "DEBUG",
        "REQUEST:",
        "REQUEST_HEADERS:",
        "REQUEST_CONTENT:",
        "RESPONSE:",
        "RESP_HEADERS:",
        "RESP_CODE:",
    )
    return entries


def validate_datetime_format(date_text: str, datetime_format: str) -> None:
    """Validate the format of a given date string against a specified datetime format.

    Parameters:
    date_text (str): The date string to be validated.
    datetime_format (str): The datetime format to which the date string should be validated.

    Raises:
    ValueError: If the date string does not match the specified datetime format.
    """
    try:
        datetime.strptime(date_text, datetime_format)
    except ValueError:
        raise ValueError("Incorrect data format, should be %Y%m%d_%H%M%S")


@pytest.fixture
def simple_data() -> tuple[dict[str, list[dict[str, str]]], str]:
    """Provide a simple data structure and its encoding for testing purposes.

    Parameters:
    None

    Returns:
    tuple: A tuple containing two elements:
        - A dictionary representing structured data with a list of dictionaries.
        - A string representing the encoding of the data.
    """
    data: dict[str, list[dict[str, str]]] = {
        "Data": [{"Name": "first_name", "Value": "John"}]
    }
    data_encoding: str = "utf-8"
    return data, data_encoding


@pytest.fixture
def client_mj30() -> Client:
    """Create and return a Mailjet API client instance for version 3.0.

    Parameters:
    None

    Returns:
    Client: An instance of the Mailjet API client configured for version 3.0. The client is authenticated using the public and private API keys provided as environment variables.
    """
    auth: tuple[str, str] = (
        os.environ["MJ_APIKEY_PUBLIC"],
        os.environ["MJ_APIKEY_PRIVATE"],
    )
    return Client(auth=auth)


@pytest.fixture
def client_mj30_invalid_auth() -> Client:
    """Create and return a Mailjet API client instance for version 3.0, but with invalid authentication credentials.

    Parameters:
    None

    Returns:
    Client: An instance of the Mailjet API client configured for version 3.0.
           The client is authenticated using invalid public and private API keys.
           If the client is used to make requests, it will raise a ValueError.
    """
    auth: tuple[str, str] = (
        "invalid_public_key",
        "invalid_private_key",
    )
    return Client(auth=auth)


@pytest.fixture
def client_mj31() -> Client:
    """Create and return a Mailjet API client instance for version 3.1.

    Parameters:
    None

    Returns:
    Client: An instance of the Mailjet API client configured for version 3.1.
           The client is authenticated using the public and private API keys provided as environment variables.

    Note:
    - The function retrieves the public and private API keys from the environment variables 'MJ_APIKEY_PUBLIC' and 'MJ_APIKEY_PRIVATE' respectively.
    - The client is initialized with the provided authentication credentials and the version set to 'v3.1'.
    """
    auth: tuple[str, str] = (
        os.environ["MJ_APIKEY_PUBLIC"],
        os.environ["MJ_APIKEY_PRIVATE"],
    )
    return Client(
        auth=auth,
        version="v3.1",
    )


def test_json_data_str_or_bytes_with_ensure_ascii(
    simple_data: tuple[dict[str, list[dict[str, str]]], str]
) -> None:
    """
    This function tests the conversion of structured data into JSON format with the specified encoding settings.

    Parameters:
    simple_data (tuple[dict[str, list[dict[str, str]]], str]): A tuple containing two elements:
        - A dictionary representing structured data with a list of dictionaries.
        - A string representing the encoding of the data.

    Returns:
    None: The function does not return any value. It performs assertions to validate the JSON conversion.
    """
    data, data_encoding = simple_data
    ensure_ascii: bool = True

    if "application/json" and data is not None:
        json_data: str | bytes | None = None
        json_data = json.dumps(data, ensure_ascii=ensure_ascii)

        assert isinstance(json_data, str)
        if not ensure_ascii:
            json_data = json_data.encode(data_encoding)
            assert isinstance(json_data, bytes)


def test_json_data_str_or_bytes_with_ensure_ascii_false(
    simple_data: tuple[dict[str, list[dict[str, str]]], str]
) -> None:
    """This function tests the conversion of structured data into JSON format with the specified encoding settings.

    It specifically tests the case where the 'ensure_ascii' parameter is set to False.

    Parameters:
    simple_data (tuple[dict[str, list[dict[str, str]]], str]): A tuple containing two elements:
        - A dictionary representing structured data with a list of dictionaries.
        - A string representing the encoding of the data.

    Returns:
    None: The function does not return any value. It performs assertions to validate the JSON conversion.
    """
    data, data_encoding = simple_data
    ensure_ascii: bool = False

    if "application/json" and data is not None:
        json_data: str | bytes | None = None
        json_data = json.dumps(data, ensure_ascii=ensure_ascii)

        assert isinstance(json_data, str)
        if not ensure_ascii:
            json_data = json_data.encode(data_encoding)
            assert isinstance(json_data, bytes)


def test_json_data_is_none(
    simple_data: tuple[dict[str, list[dict[str, str]]], str]
) -> None:
    """
    This function tests the conversion of structured data into JSON format when the data is None.

    Parameters:
    simple_data (tuple[dict[str, list[dict[str, str]]], str]): A tuple containing two elements:
        - A dictionary representing structured data with a list of dictionaries.
        - A string representing the encoding of the data.

    Returns:
    None: The function does not return any value. It performs assertions to validate the JSON conversion.
    """
    data, data_encoding = simple_data
    ensure_ascii: bool = True
    data: dict[str, list[dict[str, str]]] | None = None  # type: ignore

    if "application/json" and data is not None:
        json_data: str | bytes | None = None
        json_data = json.dumps(data, ensure_ascii=ensure_ascii)

        assert isinstance(json_data, str)
        if not ensure_ascii:
            json_data = json_data.encode(data_encoding)
            assert isinstance(json_data, bytes)


def test_prepare_url_list_splitting() -> None:
    """This function tests the prepare_url function by splitting a string containing underscores and converting the first letter of each word to uppercase.

    The function then compares the resulting list with an expected list.

    Parameters:
    None

    Note:
    - The function uses the re.sub method to replace uppercase letters with the prepare_url function.
    - It splits the resulting string into a list using the underscore as the delimiter.
    - It asserts that the resulting list is equal to the expected list ["contact", "managecontactslists"].
    """
    name: str = re.sub(r"[A-Z]", prepare_url, "contact_managecontactslists")
    split: list[str] = name.split("_")  # noqa: FURB184
    assert split == ["contact", "managecontactslists"]


def test_prepare_url_first_list_element() -> None:
    """This function tests the prepare_url function by splitting a string containing underscores, and retrieving the first element of the resulting list.

    Parameters:
    None

    Note:
    - The function uses the re.sub method to replace uppercase letters with the prepare_url function.
    - It splits the resulting string into a list using the underscore as the delimiter.
    - It asserts that the first element of the split list is equal to "contact".
    """
    name: str = re.sub(r"[A-Z]", prepare_url, "contact_managecontactslists")
    fname: str = name.split("_")[0]
    assert fname == "contact"


def test_prepare_url_headers_and_url() -> None:
    """Test the prepare_url function by splitting a string containing underscores, and retrieving the first element of the resulting list.

    Additionally, this test verifies the URL and headers generated by the prepare_url function.

    Parameters:
    None

    Note:
    - The function uses the re.sub method to replace uppercase letters with the prepare_url function.
    - It creates a Config object with the specified version and API URL.
    - It retrieves the URL and headers from the Config object using the modified string as the key.
    - It asserts that the URL is equal to "https://api.mailjet.com/v3/REST/contact" and that the headers match the expected headers.
    """
    name: str = re.sub(r"[A-Z]", prepare_url, "contact_managecontactslists")
    config: Config = Config(version="v3", api_url="https://api.mailjet.com/")
    url, headers = config[name]
    assert url == "https://api.mailjet.com/v3/REST/contact"
    assert headers == {
        "Content-type": "application/json",
        "User-agent": f"mailjet-apiv3-python/v{get_version()}",
    }


# ======= TEST CLIENT ========


def test_post_with_no_param(client_mj30: Client) -> None:
    """Tests a POST request with an empty data payload.

    This test sends a POST request to the 'create' endpoint using an empty dictionary
    as the data payload. It checks that the API responds with a 400 status code,
    indicating a bad request due to missing required parameters.

    Parameters:
        client_mj30 (Client): An instance of the Mailjet API client.

    Raises:
        AssertionError: If "StatusCode" is not in the result or if its value
        is not 400.
    """
    result = client_mj30.sender.create(data={}).json()
    assert "StatusCode" in result and result["StatusCode"] == 400


def test_get_no_param(client_mj30: Client) -> None:
    """Tests a GET request to retrieve contact data without any parameters.

    This test sends a GET request to the 'contact' endpoint without filters or
    additional parameters. It verifies that the response includes both "Data"
    and "Count" fields, confirming the endpoint returns a valid structure.

    Parameters:
        client_mj30 (Client): An instance of the Mailjet API client.

    Raises:
        AssertionError: If "Data" or "Count" are not present in the response.
    """
    result: Any = client_mj30.contact.get().json()
    assert "Data" in result and "Count" in result


def test_client_initialization_with_invalid_api_key(
    client_mj30_invalid_auth: Client,
) -> None:
    """This function tests the initialization of a Mailjet API client with invalid authentication credentials.

    Parameters:
    client_mj30_invalid_auth (Client): An instance of the Mailjet API client configured for version 3.0.
                                       The client is authenticated using invalid public and private API keys.

    Returns:
    None: The function does not return any value. It is expected to raise a ValueError when the client is used to make requests.

    Note:
    - The function uses the pytest.raises context manager to assert that a ValueError is raised when the client's contact.get() method is called.
    """
    with pytest.raises(ValueError):
        client_mj30_invalid_auth.contact.get().json()


def test_prepare_url_mixed_case_input() -> None:
    """Test prepare_url function with mixed case input.

    This function tests the prepare_url function by providing a string with mixed case characters.
    It then compares the resulting URL with the expected URL.

    Parameters:
    None

    Note:
    - The function uses the re.sub method to replace uppercase letters with the prepare_url function.
    - It creates a Config object with the specified version and API URL.
    - It retrieves the URL and headers from the Config object using the modified string as the key.
    - It asserts that the URL is equal to the expected URL and that the headers match the expected headers.
    """
    name: str = re.sub(r"[A-Z]", prepare_url, "contact")
    config: Config = Config(version="v3", api_url="https://api.mailjet.com/")
    url, headers = config[name]
    assert url == "https://api.mailjet.com/v3/REST/contact"
    assert headers == {
        "Content-type": "application/json",
        "User-agent": f"mailjet-apiv3-python/v{get_version()}",
    }


def test_prepare_url_empty_input() -> None:
    """Test prepare_url function with empty input.

    This function tests the prepare_url function by providing an empty string as input.
    It then compares the resulting URL with the expected URL.

    Parameters:
    None

    Note:
    - The function uses the re.sub method to replace uppercase letters with the prepare_url function.
    - It creates a Config object with the specified version and API URL.
    - It retrieves the URL and headers from the Config object using the modified string as the key.
    - It asserts that the URL is equal to the expected URL and that the headers match the expected headers.
    """
    name = re.sub(r"[A-Z]", prepare_url, "")
    config = Config(version="v3", api_url="https://api.mailjet.com/")
    url, headers = config[name]
    assert url == "https://api.mailjet.com/v3/REST/"
    assert headers == {
        "Content-type": "application/json",
        "User-agent": f"mailjet-apiv3-python/v{get_version()}",
    }


def test_prepare_url_with_numbers_input_bad() -> None:
    """Test the prepare_url function with input containing numbers.

    This function tests the prepare_url function by providing a string with numbers.
    It then compares the resulting URL with the expected URL.

    Parameters:
    None

    Note:
    - The function uses the re.sub method to replace uppercase letters with the prepare_url function.
    - It creates a Config object with the specified version and API URL.
    - It retrieves the URL and headers from the Config object using the modified string as the key.
    - It asserts that the URL is not equal to the expected URL and that the headers match the expected headers.
    """
    name = re.sub(r"[A-Z]", prepare_url, "contact1_managecontactslists1")
    config = Config(version="v3", api_url="https://api.mailjet.com/")
    url, headers = config[name]
    assert url != "https://api.mailjet.com/v3/REST/contact"
    assert headers == {
        "Content-type": "application/json",
        "User-agent": f"mailjet-apiv3-python/v{get_version()}",
    }


def test_prepare_url_leading_trailing_underscores_input_bad() -> None:
    """Test prepare_url function with input containing leading and trailing underscores.

    This function tests the prepare_url function by providing a string with leading and trailing underscores.
    It then compares the resulting URL with the expected URL.

    Parameters:
    None

    Note:
    - The function uses the re.sub method to replace uppercase letters with the prepare_url function.
    - It creates a Config object with the specified version and API URL.
    - It retrieves the URL and headers from the Config object using the modified string as the key.
    - It asserts that the URL is not equal to the expected URL and that the headers match the expected headers.
    """
    name: str = re.sub(r"[A-Z]", prepare_url, "_contact_managecontactslists_")
    config: Config = Config(version="v3", api_url="https://api.mailjet.com/")
    url, headers = config[name]
    assert url != "https://api.mailjet.com/v3/REST/contact"
    assert headers == {
        "Content-type": "application/json",
        "User-agent": f"mailjet-apiv3-python/v{get_version()}",
    }


def test_prepare_url_mixed_case_input_bad() -> None:
    """Test prepare_url function with mixed case input.

    This function tests the prepare_url function by providing a string with mixed case characters.
    It then compares the resulting URL with the expected URL.

    Parameters:
    None

    Note:
    - The function uses the re.sub method to replace uppercase letters with the prepare_url function.
    - It creates a Config object with the specified version and API URL.
    - It retrieves the URL and headers from the Config object using the modified string as the key.
    - It asserts that the URL is not equal to the expected URL and that the headers match the expected headers.
    """
    name: str = re.sub(r"[A-Z]", prepare_url, "cOntact")
    config: Config = Config(version="v3", api_url="https://api.mailjet.com/")
    url, headers = config[name]
    assert url != "https://api.mailjet.com/v3/REST/contact"
    assert headers == {
        "Content-type": "application/json",
        "User-agent": f"mailjet-apiv3-python/v{get_version()}",
    }


def test_debug_logging_to_stdout_has_all_debug_entries(
    client_mj30: Client,
    caplog: LogCaptureFixture,
) -> None:
    """This function tests the debug logging to stdout, ensuring that all debug entries are present.

    Parameters:
    client_mj30 (Client): An instance of the Mailjet API client.
    caplog (LogCaptureFixture): A fixture for capturing log entries.
    """
    result = client_mj30.contact.get()
    parse_response(result, lambda: logging_handler(to_file=False), debug=True)

    assert result.status_code == 200
    assert len(caplog.records) == 6
    assert all(x in caplog.text for x in debug_entries())


def test_debug_logging_to_stdout_has_all_debug_entries_when_unknown_or_not_found(
    client_mj30: Client,
    caplog: LogCaptureFixture,
) -> None:
    """This function tests the debug logging to stdout, ensuring that all debug entries are present.

    Parameters:
    client_mj30 (Client): An instance of the Mailjet API client.
    caplog (LogCaptureFixture): A fixture for capturing log entries.
    """
    # A wrong "cntact" endpoint to get 400 "Unknown resource" error message
    result = client_mj30.cntact.get()
    parse_response(result, lambda: logging_handler(to_file=False), debug=True)

    assert 400 <= result.status_code <= 404
    assert len(caplog.records) == 8
    assert all(x in caplog.text for x in debug_entries())


def test_debug_logging_to_stdout_when_retrieve_message_with_id_type_mismatch(
    client_mj30: Client,
    caplog: LogCaptureFixture,
) -> None:
    """This function tests the debug logging to stdout by retrieving message if id type mismatch, ensuring that all debug entries are present.

    GET https://api.mailjet.com/v3/REST/message/$MESSAGE_ID

    Parameters:
    client_mj30 (Client): An instance of the Mailjet API client.
    caplog (LogCaptureFixture): A fixture for capturing log entries.
    """
    _id = "*************"  # $MESSAGE_ID with all "*" will cause "Incorrect ID provided - ID type mismatch" (Error 400).
    result = client_mj30.message.get(_id)
    parse_response(result, lambda: logging_handler(to_file=False), debug=True)

    assert result.status_code == 400
    assert len(caplog.records) == 8
    assert all(x in caplog.text for x in debug_entries())


def test_debug_logging_to_stdout_when_retrieve_message_with_object_not_found(
    client_mj30: Client,
    caplog: LogCaptureFixture,
) -> None:
    """This function tests the debug logging to stdout by retrieving message if object not found, ensuring that all debug entries are present.

    GET https://api.mailjet.com/v3/REST/message/$MESSAGE_ID

    Parameters:
    client_mj30 (Client): An instance of the Mailjet API client.
    caplog (LogCaptureFixture): A fixture for capturing log entries.
    """
    _id = "0000000000000"  # $MESSAGE_ID with all zeros "0" will cause "Object not found" (Error 404).
    result = client_mj30.message.get(_id)
    parse_response(result, lambda: logging_handler(to_file=False), debug=True)

    assert result.status_code == 404
    assert len(caplog.records) == 8
    assert all(x in caplog.text for x in debug_entries())


def test_debug_logging_to_log_file(
    client_mj30: Client, caplog: LogCaptureFixture
) -> None:
    """This function tests the debug logging to a log file.

    It sends a GET request to the 'contact' endpoint of the Mailjet API client, parses the response,
    logs the debug information to a log file, validates that the log filename has the correct datetime format provided,
    and then verifies the existence and removal of the log file.

    Parameters:
    client_mj30 (Client): An instance of the Mailjet API client.
    caplog (LogCaptureFixture): A fixture for capturing log entries.
    """
    result = client_mj30.contact.get()
    parse_response(result, logging_handler, debug=True)
    partial(logging_handler, to_file=True)
    cwd = Path.cwd()
    log_files = glob.glob("*.log")
    for log_file in log_files:
        log_file_name = Path(log_file).stem
        validate_datetime_format(log_file_name, "%Y%m%d_%H%M%S")
        log_file_path = os.path.join(cwd, log_file)

        assert result.status_code == 200
        assert Path(log_file_path).exists()

        print(f"Removing log file {log_file}...")
        Path(log_file_path).unlink()
        print(f"The log file {log_file} has been removed.")



import json
import os

from mailjet_rest import Client


mailjet30 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
)

mailjet31 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
    version="v3.1",
)


def send_messages():
    """POST https://api.mailjet.com/v3.1/send"""
    # fmt: off; pylint; noqa
    data = {
        "Messages": [
            {
                "From": {"Email": "pilot@mailjet.com", "Name": "Mailjet Pilot"},
                "To": [{"Email": "passenger1@mailjet.com", "Name": "passenger 1"}],
                "Subject": "Your email flight plan!",
                "TextPart": "Dear passenger 1, welcome to Mailjet! May the "
                "delivery force be with you!",
                "HTMLPart": '<h3>Dear passenger 1, welcome to <a href="https'
                '://www.mailjet.com/">Mailjet</a>!<br />May the '
                "delivery force be with you!",
            },
        ],
        "SandboxMode": True,  # Remove to send real message.
    }
    # fmt: on; pylint; noqa
    return mailjet31.send.create(data=data)


def retrieve_messages_from_campaign():
    """GET https://api.mailjet.com/v3/REST/message?CampaignID=$CAMPAIGNID"""
    filters = {
        "CampaignID": "*****",  # Put real ID to make it work.
    }
    return mailjet30.message.get(filters=filters)


def retrieve_message():
    """GET https://api.mailjet.com/v3/REST/message/$MESSAGE_ID"""
    _id = "*****************"  # Put real ID to make it work.
    return mailjet30.message.get(_id)


def view_message_history():
    """GET https://api.mailjet.com/v3/REST/messagehistory/$MESSAGE_ID"""
    _id = "*****************"  # Put real ID to make it work.
    return mailjet30.messagehistory.get(_id)


def retrieve_statistic():
    """GET https://api.mailjet.com/v3/REST/statcounters?CounterSource=APIKey
    \\&CounterTiming=Message\\&CounterResolution=Lifetime
    """
    filters = {
        "CounterSource": "APIKey",
        "CounterTiming": "Message",
        "CounterResolution": "Lifetime",
    }
    return mailjet30.statcounters.get(filters=filters)


if __name__ == "__main__":
    result = retrieve_statistic()
    print(result.status_code)
    try:
        print(json.dumps(result.json(), indent=4))
    except json.decoder.JSONDecodeError:
        print(result.text)



import json
import os

from mailjet_rest import Client


mailjet30 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
)

mailjet31 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
    version="v3.1",
)


def basic_setup():
    """POST https://api.mailjet.com/v3/REST/parseroute"""
    data = {"Url": "https://www.mydomain.com/mj_parse.php"}
    return mailjet30.parseroute.create(data=data)


if __name__ == "__main__":
    result = basic_setup()
    print(result.status_code)
    try:
        print(json.dumps(result.json(), indent=4))
    except json.decoder.JSONDecodeError:
        print(result.text)



import json
import os

from mailjet_rest import Client


mailjet30 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
)

mailjet31 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
    version="v3.1",
)


def validate_an_entire_domain():
    """GET https: // api.mailjet.com / v3 / REST / dns"""
    _id = "$dns_ID"
    return mailjet30.dns.get(id=_id)


def do_an_immediate_check_via_a_post():
    """POST https://api.mailjet.com/v3/REST/dns/$dns_ID/check"""
    _id = "$dns_ID"
    return mailjet30.dns_check.create(id=_id)


def host_a_text_file():
    """GET https://api.mailjet.com/v3/REST/sender"""
    _id = "$sender_ID"
    return mailjet30.sender.get(id=_id)


def validation_by_doing_a_post():
    """POST https://api.mailjet.com/v3/REST/sender/$sender_ID/validate"""
    _id = "$sender_ID"
    return mailjet30.sender_validate.create(id=_id)


def spf_and_dkim_validation():
    """ET https://api.mailjet.com/v3/REST/dns"""
    _id = "$dns_ID"
    return mailjet30.dns.get(id=_id)


def use_a_sender_on_all_api_keys():
    """POST https://api.mailjet.com/v3/REST/metasender"""
    data = {
        "Description": "Metasender 1 - used for Promo emails",
        "Email": "pilot@mailjet.com",
    }
    return mailjet30.metasender.create(data=data)


if __name__ == "__main__":
    result = validate_an_entire_domain()
    print(result.status_code)
    try:
        print(json.dumps(result.json(), indent=4))
    except json.decoder.JSONDecodeError:
        print(result.text)



import json
import os

from mailjet_rest import Client


mailjet30 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
)

mailjet31 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
    version="v3.1",
)


def create_a_campaign_draft():
    """POST https://api.mailjet.com/v3/REST/campaigndraft"""
    data = {
        "Locale": "en_US",
        "Sender": "MisterMailjet",
        "SenderEmail": "Mister@mailjet.com",
        "Subject": "Greetings from Mailjet",
        "ContactsListID": "$ID_CONTACTSLIST",
        "Title": "Friday newsletter",
    }
    return mailjet30.campaigndraft.create(data=data)


def by_adding_custom_content():
    """POST https://api.mailjet.com/v3/REST/campaigndraft/$draft_ID/detailcontent"""
    _id = "$draft_ID"
    data = {
        "Headers": "object",
        "Html-part": "<h3>Dear passenger, welcome to Mailjet!</h3><br />May the delivery force be with you!",
        "MJMLContent": "",
        "Text-part": "Dear passenger, welcome to Mailjet! May the delivery force be with you!",
    }
    return mailjet30.campaigndraft_detailcontent.create(id=_id, data=data)


def test_your_campaign():
    """POST https://api.mailjet.com/v3/REST/campaigndraft/$draft_ID/test"""
    _id = "$draft_ID"
    data = {"Recipients": [{"Email": "passenger@mailjet.com", "Name": "Passenger 1"}]}
    return mailjet30.campaigndraft_test.create(id=_id, data=data)


def schedule_the_sending():
    """POST https://api.mailjet.com/v3/REST/campaigndraft/$draft_ID/schedule"""
    _id = "$draft_ID"
    data = {"Date": "2018-01-01T00:00:00"}
    return mailjet30.campaigndraft_schedule.create(id=_id, data=data)


def send_the_campaign_right_away():
    """POST https://api.mailjet.com/v3/REST/campaigndraft/$draft_ID/send"""
    _id = "$draft_ID"
    return mailjet30.campaigndraft_send.create(id=_id)


def api_call_requirements():
    """POST https://api.mailjet.com/v3.1/send"""
    # fmt: off
    data = {
        "Messages": [
            {
                "From": {
                    "Email": "pilot@mailjet.com",
                    "Name": "Mailjet Pilot"},
                "To": [
                    {
                        "Email": "passenger1@mailjet.com",
                        "Name": "passenger 1"}],
                "Subject": "Your email flight plan!",
                "TextPart": "Dear passenger 1, welcome to Mailjet! May the delivery force be with you!",
                "HTMLPart": "<h3>Dear passenger 1, welcome to <a "
                            "href=\"https://www.mailjet.com/\">Mailjet</a>!</h3><br />May the delivery force be with "
                            "you!",
                "CustomCampaign": "SendAPI_campaign",
                "DeduplicateCampaign": True}]}
    # fmt: on
    return mailjet31.send.create(data=data)


if __name__ == "__main__":
    result = create_a_campaign_draft()
    print(result.status_code)
    try:
        print(json.dumps(result.json(), indent=4))
    except json.decoder.JSONDecodeError:
        print(result.text)



from __future__ import annotations

import pytest

from mailjet_rest.utils.version import get_version, VERSION


def test_version_length_equal_three() -> None:
    """Verify that the tuple contains 3 items."""
    assert len(VERSION) == 3


def test_get_version_is_none() -> None:
    """Test that package version is none."""
    version: None = None
    result: str | tuple[int, ...]
    result = get_version(version)
    assert isinstance(result, str)
    result = tuple(map(int, result.split(".")))
    assert result == VERSION
    assert isinstance(result, tuple)


def test_get_version() -> None:
    """Test that package version is string.

    Verify that if it's equal to tuple after splitting and mapped to tuple.
    """
    result: str | tuple[int, ...]
    result = get_version(VERSION)
    assert isinstance(result, str)
    result = tuple(map(int, result.split(".")))
    assert result == VERSION
    assert isinstance(result, tuple)


def test_get_version_raises_exception() -> None:
    """Test that package version raise ValueError if its length is not equal 3."""
    version: tuple[int, int] = (
        1,
        2,
    )
    with pytest.raises(ValueError):
        get_version(version)



"""This module provides the main client and helper classes for interacting with the Mailjet API.

The `mailjet_rest.client` module includes the core `Client` class for managing
API requests, configuration, and error handling, as well as utility functions
and classes for building request headers, URLs, and parsing responses.

Classes:
    - Config: Manages configuration settings for the Mailjet API.
    - Endpoint: Represents specific API endpoints and provides methods for
      common HTTP operations like GET, POST, PUT, and DELETE.
    - Client: The main API client for authenticating and making requests.
    - ApiError: Base class for handling API-specific errors, with subclasses
      for more specific error types (e.g., `AuthorizationError`, `TimeoutError`).

Functions:
    - prepare_url: Prepares URLs for API requests.
    - api_call: A helper function that sends HTTP requests to the API and handles
      responses.
    - build_headers: Builds HTTP headers for the requests.
    - build_url: Constructs the full API URL based on endpoint and parameters.
    - parse_response: Parses API responses and handles error conditions.

Exceptions:
    - ApiError: Base exception for API errors, with subclasses to represent
      specific error types, such as `AuthorizationError`, `TimeoutError`,
      `ActionDeniedError`, and `ValidationError`.
"""

from __future__ import annotations

import json
import logging
import re
import sys
from datetime import datetime
from datetime import timezone
from re import Match
from typing import TYPE_CHECKING
from typing import Any

import requests  # type: ignore[import-untyped]
from requests.compat import urljoin  # type: ignore[import-untyped]

from mailjet_rest.utils.version import get_version


if TYPE_CHECKING:
    from collections.abc import Callable
    from collections.abc import Mapping

    from requests.models import Response  # type: ignore[import-untyped]


requests.packages.urllib3.disable_warnings()  # type: ignore[attr-defined]


def prepare_url(key: Match[str]) -> str:
    """Replace capital letters in the input string with a dash prefix and converts them to lowercase.

    Parameters:
    key (Match[str]): A match object representing a substring from the input string. The substring should contain a single capital letter.

    Returns:
    str: A string containing a dash followed by the lowercase version of the input capital letter.
    """
    char_elem = key.group(0)
    if char_elem.isupper():
        return "-" + char_elem.lower()
    return ""


class Config:
    """Configuration settings for interacting with the Mailjet API.

    This class stores and manages API configuration details, including the API URL,
    version, and user agent string. It provides methods for initializing these settings
    and generating endpoint-specific URLs and headers as required for API interactions.

    Attributes:
        DEFAULT_API_URL (str): The default base URL for Mailjet API requests.
        API_REF (str): Reference URL for Mailjet's API documentation.
        version (str): API version to use, defaulting to 'v3'.
        user_agent (str): User agent string including the package version for tracking.
    """

    DEFAULT_API_URL: str = "https://api.mailjet.com/"
    API_REF: str = "https://dev.mailjet.com/email-api/v3/"
    version: str = "v3"
    user_agent: str = "mailjet-apiv3-python/v" + get_version()

    def __init__(self, version: str | None = None, api_url: str | None = None) -> None:
        """Initialize a new Config instance with specified or default API settings.

        This initializer sets the API version and base URL. If no version or URL
        is provided, it defaults to the predefined class values.

        Parameters:
        - version (str | None): The API version to use. If None, the default version ('v3') is used.
        - api_url (str | None): The base URL for API requests. If None, the default URL (DEFAULT_API_URL) is used.
        """
        if version is not None:
            self.version = version
        self.api_url = api_url or self.DEFAULT_API_URL

    def __getitem__(self, key: str) -> tuple[str, dict[str, str]]:
        """Retrieve the API endpoint URL and headers for a given key.

        This method builds the URL and headers required for specific API interactions.
        The URL is adjusted based on the API version, and additional headers are
        appended depending on the endpoint type. Specific keys modify content-type
        for endpoints expecting CSV or plain text.

        Parameters:
        - key (str): The name of the API endpoint, which influences URL structure and header configuration.

        Returns:
        - tuple[str, dict[str, str]]: A tuple containing the constructed URL and headers required for the specified endpoint.

        Examples:
            For the "contactslist_csvdata" key, a URL pointing to 'DATA/' and a
            'Content-type' of 'text/plain' is returned.
            For the "batchjob_csverror" key, a URL with 'DATA/' and a 'Content-type'
            of 'text/csv' is returned.
        """
        # Append version to URL.
        # Forward slash is ignored if present in self.version.
        url = urljoin(self.api_url, self.version + "/")
        headers: dict[str, str] = {
            "Content-type": "application/json",
            "User-agent": self.user_agent,
        }
        if key.lower() == "contactslist_csvdata":
            url = urljoin(url, "DATA/")
            headers["Content-type"] = "text/plain"
        elif key.lower() == "batchjob_csverror":
            url = urljoin(url, "DATA/")
            headers["Content-type"] = "text/csv"
        elif key.lower() != "send" and self.version != "v4":
            url = urljoin(url, "REST/")
        url += key.split("_")[0].lower()
        return url, headers


class Endpoint:
    """A class representing a specific Mailjet API endpoint.

    This class provides methods to perform HTTP requests to a given API endpoint,
    including GET, POST, PUT, and DELETE requests. It manages URL construction,
    headers, and authentication for interacting with the endpoint.

    Attributes:
    - _url (str): The base URL of the endpoint.
    - headers (dict[str, str]): The headers to be included in API requests.
    - _auth (tuple[str, str] | None): The authentication credentials.
    - action (str | None): The specific action to be performed on the endpoint.

    Methods:
    - _get: Internal method to perform a GET request.
    - get_many: Performs a GET request to retrieve multiple resources.
    - get: Performs a GET request to retrieve a specific resource.
    - create: Performs a POST request to create a new resource.
    - update: Performs a PUT request to update an existing resource.
    - delete: Performs a DELETE request to delete a resource.
    """

    def __init__(
        self,
        url: str,
        headers: dict[str, str],
        auth: tuple[str, str] | None,
        action: str | None = None,
    ) -> None:
        """Initialize a new Endpoint instance.

        Args:
            url (str): The base URL for the endpoint.
            headers (dict[str, str]): Headers for API requests.
            auth (tuple[str, str] | None): Authentication credentials.
            action (str | None): Action to perform on the endpoint, if any.
        """
        self._url, self.headers, self._auth, self.action = url, headers, auth, action

    def _get(
        self,
        filters: Mapping[str, str | Any] | None = None,
        action_id: str | None = None,
        id: str | None = None,
        **kwargs: Any,
    ) -> Response:
        """Perform an internal GET request to the endpoint.

        Constructs the URL with the provided filters and action_id to retrieve
        specific data from the API.

        Parameters:
        - filters (Mapping[str, str | Any] | None): Filters to be applied in the request.
        - action_id (str | None): The specific action ID for the endpoint to be performed.
        - id (str | None): The ID of the specific resource to be retrieved.
        - **kwargs (Any): Additional keyword arguments to be passed to the API call.

        Returns:
        - Response: The response object from the API call.
        """
        return api_call(
            self._auth,
            "get",
            self._url,
            headers=self.headers,
            action=self.action,
            action_id=action_id,
            filters=filters,
            resource_id=id,
            **kwargs,
        )

    def get_many(
        self,
        filters: Mapping[str, str | Any] | None = None,
        action_id: str | None = None,
        **kwargs: Any,
    ) -> Response:
        """Perform a GET request to retrieve multiple resources.

        Parameters:
        - filters (Mapping[str, str | Any] | None): Filters to be applied in the request.
        - action_id (str | None): The specific action ID to be performed.
        - **kwargs (Any): Additional keyword arguments to be passed to the API call.

        Returns:
        - Response: The response object from the API call containing multiple resources.
        """
        return self._get(filters=filters, action_id=action_id, **kwargs)

    def get(
        self,
        id: str | None = None,
        filters: Mapping[str, str | Any] | None = None,
        action_id: str | None = None,
        **kwargs: Any,
    ) -> Response:
        """Perform a GET request to retrieve a specific resource.

        Parameters:
        - id (str | None): The ID of the specific resource to be retrieved.
        - filters (Mapping[str, str | Any] | None): Filters to be applied in the request.
        - action_id (str | None): The specific action ID to be performed.
        - **kwargs (Any): Additional keyword arguments to be passed to the API call.

        Returns:
        - Response: The response object from the API call containing the specific resource.
        """
        return self._get(id=id, filters=filters, action_id=action_id, **kwargs)

    def create(
        self,
        data: str | bytes | dict[Any, Any] | None = None,
        filters: Mapping[str, str | Any] | None = None,
        id: str | None = None,
        action_id: str | None = None,
        ensure_ascii: bool = True,
        data_encoding: str = "utf-8",
        **kwargs: Any,
    ) -> Response:
        """Perform a POST request to create a new resource.

        Parameters:
        - data (str | bytes | dict[Any, Any] | None): The data to include in the request body.
        - filters (Mapping[str, str | Any] | None): Filters to be applied in the request.
        - id (str | None): The ID of the specific resource to be created.
        - action_id (str | None): The specific action ID to be performed.
        - ensure_ascii (bool): Whether to ensure ASCII characters in the data.
        - data_encoding (str): The encoding to be used for the data.
        - **kwargs (Any): Additional keyword arguments to be passed to the API call.

        Returns:
        - Response: The response object from the API call.
        """
        if self.headers.get("Content-type") == "application/json" and data is not None:
            data = json.dumps(
                data,
                ensure_ascii=ensure_ascii,
            )
            if not ensure_ascii:
                data = data.encode(data_encoding)
        return api_call(
            self._auth,
            "post",
            self._url,
            headers=self.headers,
            resource_id=id,
            data=data,  # type: ignore[arg-type]
            action=self.action,
            action_id=action_id,
            filters=filters,
            **kwargs,
        )

    def update(
        self,
        id: str | None,
        data: dict | None = None,
        filters: Mapping[str, str | Any] | None = None,
        action_id: str | None = None,
        ensure_ascii: bool = True,
        data_encoding: str = "utf-8",
        **kwargs: Any,
    ) -> Response:
        """Perform a PUT request to update an existing resource.

        Parameters:
        - id (str | None): The ID of the specific resource to be updated.
        - data (dict | None): The data to be sent in the request body.
        - filters (Mapping[str, str | Any] | None): Filters to be applied in the request.
        - action_id (str | None): The specific action ID to be performed.
        - ensure_ascii (bool): Whether to ensure ASCII characters in the data.
        - data_encoding (str): The encoding to be used for the data.
        - **kwargs (Any): Additional keyword arguments to be passed to the API call.

        Returns:
        - Response: The response object from the API call.
        """
        json_data: str | bytes | None = None
        if self.headers.get("Content-type") == "application/json" and data is not None:
            json_data = json.dumps(data, ensure_ascii=ensure_ascii)
            if not ensure_ascii:
                json_data = json_data.encode(data_encoding)
        return api_call(
            self._auth,
            "put",
            self._url,
            resource_id=id,
            headers=self.headers,
            data=json_data,
            action=self.action,
            action_id=action_id,
            filters=filters,
            **kwargs,
        )

    def delete(self, id: str | None, **kwargs: Any) -> Response:
        """Perform a DELETE request to delete a resource.

        Parameters:
        - id (str | None): The ID of the specific resource to be deleted.
        - **kwargs (Any): Additional keyword arguments to be passed to the API call.

        Returns:
        - Response: The response object from the API call.
        """
        return api_call(
            self._auth,
            "delete",
            self._url,
            action=self.action,
            headers=self.headers,
            resource_id=id,
            **kwargs,
        )


class Client:
    """A client for interacting with the Mailjet API.

    This class manages authentication, configuration, and API endpoint access.
    It initializes with API authentication details and uses dynamic attribute access
    to allow flexible interaction with various Mailjet API endpoints.

    Attributes:
    - auth  (tuple[str, str] | None): A tuple containing the API key and secret for authentication.
    - config (Config): An instance of the Config class, which holds API configuration settings.

    Methods:
    - __init__: Initializes a new Client instance with authentication and configuration settings.
    - __getattr__: Handles dynamic attribute access, allowing for accessing API endpoints as attributes.
    """

    def __init__(self, auth: tuple[str, str] | None = None, **kwargs: Any) -> None:
        """Initialize a new Client instance for API interaction.

        This method sets up API authentication and configuration. The `auth` parameter
        provides a tuple with the API key and secret. Additional keyword arguments can
        specify configuration options like API version and URL.

        Parameters:
        - auth (tuple[str, str] | None): A tuple containing the API key and secret for authentication. If None, authentication is not required.
        - **kwargs (Any): Additional keyword arguments, such as `version` and `api_url`, for configuring the client.

        Example:
            client = Client(auth=("api_key", "api_secret"), version="v3")
        """
        self.auth = auth
        version: str | None = kwargs.get("version")
        api_url: str | None = kwargs.get("api_url")
        self.config = Config(version=version, api_url=api_url)

    def __getattr__(self, name: str) -> Any:
        """Dynamically access API endpoints as attributes.

        This method allows for flexible, attribute-style access to API endpoints.
        It constructs the appropriate endpoint URL and headers based on the attribute
        name, which it parses to identify the resource and optional sub-resources.

        Parameters:
        - name (str): The name of the attribute being accessed, corresponding to the Mailjet API endpoint.


        Returns:
        - Endpoint: An instance of the `Endpoint` class, initialized with the constructed URL, headers, action, and authentication details.
        """
        name_regex: str = re.sub(r"[A-Z]", prepare_url, name)
        split: list[str] = name_regex.split("_")  # noqa: RUF100
        # identify the resource
        fname: str = split[0]
        action: str | None = None
        if len(split) > 1:
            # identify the sub resource (action)
            action = split[1]
            if action == "csvdata":
                action = "csvdata/text:plain"
            if action == "csverror":
                action = "csverror/text:csv"
        url, headers = self.config[name]
        return type(fname, (Endpoint,), {})(
            url=url,
            headers=headers,
            action=action,
            auth=self.auth,
        )


def api_call(
    auth: tuple[str, str] | None,
    method: str,
    url: str,
    headers: dict[str, str],
    data: str | bytes | None = None,
    filters: Mapping[str, str | Any] | None = None,
    resource_id: str | None = None,
    timeout: int = 60,
    debug: bool = False,
    action: str | None = None,
    action_id: str | None = None,
    **kwargs: Any,
) -> Response | Any:
    """Make an API call to a specified URL using the provided method, headers, and other parameters.

    Parameters:
    - auth (tuple[str, str] | None): A tuple containing the API key and secret for authentication.
    - method (str): The HTTP method to be used for the API call (e.g., 'get', 'post', 'put', 'delete').
    - url (str): The URL to which the API call will be made.
    - headers (dict[str, str]): A dictionary containing the headers to be included in the API call.
    - data (str | bytes | None): The data to be sent in the request body.
    - filters (Mapping[str, str | Any] | None): A dictionary containing filters to be applied in the request.
    - resource_id (str | None): The ID of the specific resource to be accessed.
    - timeout (int): The timeout for the API call in seconds.
    - debug (bool): A flag indicating whether debug mode is enabled.
    - action (str | None): The specific action to be performed on the resource.
    - action_id (str | None): The ID of the specific action to be performed.
    - **kwargs (Any): Additional keyword arguments to be passed to the API call.

    Returns:
    - Response | Any: The response object from the API call if the request is successful, or an exception if an error occurs.
    """
    url = build_url(
        url,
        method=method,
        action=action,
        resource_id=resource_id,
        action_id=action_id,
    )
    req_method = getattr(requests, method)

    try:
        filters_str: str | None = None
        if filters:
            filters_str = "&".join(f"{k}={v}" for k, v in filters.items())
        response = req_method(
            url,
            data=data,
            params=filters_str,
            headers=headers,
            auth=auth,
            timeout=timeout,
            verify=True,
            stream=False,
        )

    except requests.exceptions.Timeout:
        raise TimeoutError
    except requests.RequestException as e:
        raise ApiError(e)  # noqa: RUF100, B904
    except Exception:
        raise
    else:
        return response


def build_headers(
    resource: str,
    action: str,
    extra_headers: dict[str, str] | None = None,
) -> dict[str, str]:
    """Build headers based on resource and action.

    Parameters:
    - resource (str): The name of the resource for which headers are being built.
    - action (str): The specific action being performed on the resource.
    - extra_headers (dict[str, str] | None): Additional headers to be included in the request. Defaults to None.

    Returns:
    - dict[str, str]: A dictionary containing the headers to be included in the API request.
    """
    headers: dict[str, str] = {"Content-type": "application/json"}

    if resource.lower() == "contactslist" and action.lower() == "csvdata":
        headers = {"Content-type": "text/plain"}
    elif resource.lower() == "batchjob" and action.lower() == "csverror":
        headers = {"Content-type": "text/csv"}

    if extra_headers:
        headers.update(extra_headers)

    return headers


def build_url(
    url: str,
    method: str | None,
    action: str | None = None,
    resource_id: str | None = None,
    action_id: str | None = None,
) -> str:
    """Construct a URL for making an API request.

    This function takes the base URL, method, action, resource ID, and action ID as parameters
    and constructs a URL by appending the resource ID, action, and action ID to the base URL.

    Parameters:
    url (str): The base URL for the API request.
    method (str | None): The HTTP method for the API request (e.g., 'get', 'post', 'put', 'delete').
    action (str | None): The specific action to be performed on the resource. Defaults to None.
    resource_id (str | None): The ID of the specific resource to be accessed. Defaults to None.
    action_id (str | None): The ID of the specific action to be performed. Defaults to None.

    Returns:
    str: The constructed URL for the API request.
    """
    if resource_id:
        url += f"/{resource_id}"
    if action:
        url += f"/{action}"
        if action_id:
            url += f"/{action_id}"
    return url


def logging_handler(
    to_file: bool = False,
) -> logging.Logger:
    """Create and configure a logger for logging API requests.

    This function creates a logger object and configures it to handle both
    standard output (stdout) and a file if the `to_file` parameter is set to True.
    The logger is set to log at the DEBUG level and uses a custom formatter to
    include the log level and message.

    Parameters:
    to_file (bool): A flag indicating whether to log to a file. If True, logs will be written to a file.
                     Defaults to False.

    Returns:
    logging.Logger: A configured logger object for logging API requests.
    """
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    formatter = logging.Formatter("%(levelname)s | %(message)s")

    if to_file:
        now = datetime.now(tz=timezone.utc)
        date_time = now.strftime("%Y%m%d_%H%M%S")

        log_file = f"{date_time}.log"
        file_handler = logging.FileHandler(log_file)
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)

    stdout_handler = logging.StreamHandler(sys.stdout)
    stdout_handler.setFormatter(formatter)
    logger.addHandler(stdout_handler)

    return logger


def parse_response(
    response: Response,
    log: Callable,
    debug: bool = False,
) -> Any:
    """Parse the response from an API request and return the JSON data.

    Parameters:
    response (Response): The response object from the API request.
    log (Callable): A function or method that logs debug information.
    debug (bool): A flag indicating whether debug mode is enabled. Defaults to False.

    Returns:
    Any: The JSON data from the API response.
    """
    data = response.json()

    if debug:
        lgr = log()
        lgr.debug("REQUEST: %s", response.request.url)
        lgr.debug("REQUEST_HEADERS: %s", response.request.headers)
        lgr.debug("REQUEST_CONTENT: %s", response.request.body)

        lgr.debug("RESPONSE: %s", response.content)
        lgr.debug("RESP_HEADERS: %s", response.headers)
        lgr.debug("RESP_CODE: %s", response.status_code)
        # Clear logger handlers to prevent making log duplications
        logging.getLogger().handlers.clear()

    return data


class ApiError(Exception):
    """Base class for all API-related errors.

    This exception serves as the root for all custom API error types,
    allowing for more specific error handling based on the type of API
    failure encountered.
    """


class AuthorizationError(ApiError):
    """Error raised for authorization failures.

    This error is raised when the API request fails due to invalid
    or missing authentication credentials.
    """


class ActionDeniedError(ApiError):
    """Error raised when an action is denied by the API.

    This exception is triggered when an action is requested but is not
    permitted, likely due to insufficient permissions.
    """


class CriticalApiError(ApiError):
    """Error raised for critical API failures.

    This error represents severe issues with the API or infrastructure
    that prevent requests from completing.
    """


class ApiRateLimitError(ApiError):
    """Error raised when the API rate limit is exceeded.

    This exception is raised when the user has made too many requests
    within a given time frame, as enforced by the API's rate limit policy.
    """


class TimeoutError(ApiError):
    """Error raised when an API request times out.

    This error is raised if an API request does not complete within
    the allowed timeframe, possibly due to network issues or server load.
    """


class DoesNotExistError(ApiError):
    """Error raised when a requested resource does not exist.

    This exception is triggered when a specific resource is requested
    but cannot be found in the API, indicating a potential data mismatch
    or invalid identifier.
    """


class ValidationError(ApiError):
    """Error raised for invalid input data.

    This exception is raised when the input data for an API request
    does not meet validation requirements, such as incorrect data types
    or missing fields.
    """



"""A suite of tests for Mailjet API client functionality."""

import os
import random
import string
import unittest
from pathlib import Path
from typing import Any
from typing import ClassVar

from mailjet_rest import Client


class TestSuite(unittest.TestCase):
    """A suite of tests for Mailjet API client functionality.

    This class provides setup and teardown functionality for tests involving the
    Mailjet API client, with authentication and client initialization handled
    in `setUp`. Each test in this suite operates with the configured Mailjet client
    instance to simulate API interactions.
    """

    def setUp(self) -> None:
        """Set up the test environment by initializing authentication credentials and the Mailjet client.

        This method is called before each test to ensure a consistent testing
        environment. It retrieves the API keys from environment variables and
        uses them to create an instance of the Mailjet `Client` for authenticated
        API interactions.

        Attributes:
        - self.auth (tuple[str, str]): A tuple containing the public and private API keys obtained from the environment variables 'MJ_APIKEY_PUBLIC' and 'MJ_APIKEY_PRIVATE' respectively.
        - self.client (Client):  An instance of the Mailjet Client class, initialized with the provided authentication credentials.
        """
        self.auth: tuple[str, str] = (
            os.environ["MJ_APIKEY_PUBLIC"],
            os.environ["MJ_APIKEY_PRIVATE"],
        )
        self.client: Client = Client(auth=self.auth)

    def test_get_no_param(self) -> None:
        """This test function sends a GET request to the Mailjet API endpoint for contacts without any parameters.

        It verifies that the response contains 'Data' and 'Count' fields.

        Parameters:
        None
        """
        result: Any = self.client.contact.get().json()
        self.assertTrue("Data" in result and "Count" in result)

    def test_get_valid_params(self) -> None:
        """This test function sends a GET request to the Mailjet API endpoint for contacts with a valid parameter 'limit'.

        It verifies that the response contains a count of contacts that is within the range of 0 to 2.

        Parameters:
        None
        """
        result: Any = self.client.contact.get(filters={"limit": 2}).json()
        self.assertTrue(result["Count"] >= 0 or result["Count"] <= 2)

    def test_get_invalid_parameters(self) -> None:
        """This test function sends a GET request to the Mailjet API endpoint for contacts with an invalid parameter.

        It verifies that the response contains 'Count' field, demonstrating that invalid parameters are ignored.

        Parameters:
        None
        """
        # invalid parameters are ignored
        result: Any = self.client.contact.get(filters={"invalid": "false"}).json()
        self.assertTrue("Count" in result)

    def test_get_with_data(self) -> None:
        """This test function sends a GET request to the Mailjet API endpoint for contacts with 'data' parameter.

        It verifies that the request is successful (status code 200) and does not use the 'data' parameter.

        Parameters:
        None
        """
        # it shouldn't use data
        result = self.client.contact.get(data={"Email": "api@mailjet.com"})
        self.assertTrue(result.status_code == 200)

    def test_get_with_action(self) -> None:
        """This function tests the functionality of adding a contact to a contact list using the Mailjet API client.

        It first retrieves a contact and a contact list from the API, then adds the contact to the list.
        Finally, it verifies that the contact has been successfully added to the list.

        Parameters:
        None

        Attributes:
        - get_contact (Any): The result of the initial contact retrieval, containing a single contact.
        - contact_id (str): The ID of the retrieved contact.
        - post_contact (Response): The response from creating a new contact if no contact was found.
        - get_contact_list (Any): The result of the contact list retrieval, containing a single contact list.
        - list_id (str): The ID of the retrieved contact list.
        - post_contact_list (Response): The response from creating a new contact list if no contact list was found.
        - data (dict[str, list[dict[str, str]]]): The data for managing contact lists, containing the list ID and action to add the contact.
        - result_add_list (Response): The response from adding the contact to the contact list.
        - result (Any): The result of retrieving the contact's contact lists, containing the count of contact lists.
        """
        get_contact: Any = self.client.contact.get(filters={"limit": 1}).json()
        if get_contact["Count"] != 0:
            contact_id: str = get_contact["Data"][0]["ID"]
        else:
            contact_random_email: str = (
                "".join(
                    random.choice(string.ascii_uppercase + string.digits)
                    for _ in range(10)
                )
                + "@mailjet.com"
            )
            post_contact = self.client.contact.create(
                data={"Email": contact_random_email},
            )
            self.assertTrue(post_contact.status_code == 201)
            contact_id = post_contact.json()["Data"][0]["ID"]

        get_contact_list: Any = self.client.contactslist.get(
            filters={"limit": 1},
        ).json()
        if get_contact_list["Count"] != 0:
            list_id: str = get_contact_list["Data"][0]["ID"]
        else:
            contact_list_random_name: str = (
                "".join(
                    random.choice(string.ascii_uppercase + string.digits)
                    for _ in range(10)
                )
                + "@mailjet.com"
            )
            post_contact_list = self.client.contactslist.create(
                data={"Name": contact_list_random_name},
            )
            self.assertTrue(post_contact_list.status_code == 201)
            list_id = post_contact_list.json()["Data"][0]["ID"]

        data: dict[str, list[dict[str, str]]] = {
            "ContactsLists": [{"ListID": list_id, "Action": "addnoforce"}],
        }
        result_add_list = self.client.contact_managecontactslists.create(
            id=contact_id,
            data=data,
        )
        self.assertTrue(result_add_list.status_code == 201)

        result = self.client.contact_getcontactslists.get(contact_id).json()
        self.assertTrue("Count" in result)

    def test_get_with_id_filter(self) -> None:
        """This test function sends a GET request to the Mailjet API endpoint for contacts with a specific email address obtained from a previous contact retrieval.

        It verifies that the response contains a contact with the same email address as the one used in the filter.

        Parameters:
        None

        Attributes:
        - result_contact (Any): The result of the initial contact retrieval, containing a single contact.
        - result_contact_with_id (Any): The result of the contact retrieval using the email address from the initial contact as a filter.
        """
        result_contact: Any = self.client.contact.get(filters={"limit": 1}).json()
        result_contact_with_id: Any = self.client.contact.get(
            filter={"Email": result_contact["Data"][0]["Email"]},
        ).json()
        self.assertTrue(
            result_contact_with_id["Data"][0]["Email"]
            == result_contact["Data"][0]["Email"],
        )

    def test_post_with_no_param(self) -> None:
        """This function tests the behavior of the Mailjet API client when attempting to create a sender with no parameters.

        The function sends a POST request to the Mailjet API endpoint for creating a sender with an empty
        data dictionary. It then verifies that the response contains a 'StatusCode' field with a value of 400,
        indicating a bad request. This test ensures that the client handles missing required parameters
        appropriately.

        Parameters:
        None
        """
        result: Any = self.client.sender.create(data={}).json()
        self.assertTrue("StatusCode" in result and result["StatusCode"] == 400)

    def test_client_custom_version(self) -> None:
        """This test function verifies the functionality of setting a custom version for the Mailjet API client.

        The function initializes a new instance of the Mailjet Client with custom version "v3.1".
        It then asserts that the client's configuration version is correctly set to "v3.1".
        Additionally, it verifies that the send endpoint URL in the client's configuration is updated to the correct version.

        Parameters:
        None
        """
        self.client = Client(auth=self.auth, version="v3.1")
        self.assertEqual(self.client.config.version, "v3.1")
        self.assertEqual(
            self.client.config["send"][0],
            "https://api.mailjet.com/v3.1/send",
        )

    def test_user_agent(self) -> None:
        """This function tests the user agent configuration of the Mailjet API client.

        The function initializes a new instance of the Mailjet Client with a custom version "v3.1".
        It then asserts that the client's user agent is correctly set to "mailjet-apiv3-python/v1.3.5".
        This test ensures that the client's user agent is properly configured and includes the correct version information.

        Parameters:
        None
        """
        self.client = Client(auth=self.auth, version="v3.1")
        self.assertEqual(self.client.config.user_agent, "mailjet-apiv3-python/v1.5.1")


class TestCsvImport(unittest.TestCase):
    """Tests for Mailjet API csv import functionality.

    This class provides setup and teardown functionality for tests involving the
    csv import functionality, with authentication and client initialization handled
    in `setUp`. Each test in this suite operates with the configured Mailjet client
    instance to simulate API interactions.

    Attributes:
    - _shared_state (dict[str, str]): A dictionary containing values taken from tests to share them in other tests.
    """

    _shared_state: ClassVar[dict[str, Any]] = {}

    @classmethod
    def get_shared(cls, key: str) -> Any:
        """Retrieve a value from shared test state.

        Parameters:
        - key (str): The key to look up in shared state.

        Returns:
        - Any: The stored value, or None if key doesn't exist.
        """
        return cls._shared_state.get(key)

    @classmethod
    def set_shared(cls, key: str, value: Any) -> None:
        """Store a value in shared test state.

        Parameters:
        - key (str): The key to store the value under.
        - value (Any): The value to store.
        """
        cls._shared_state[key] = value

    def setUp(self) -> None:
        """Set up the test environment by initializing authentication credentials and the Mailjet client.

        This method is called before each test to ensure a consistent testing
        environment. It retrieves the API keys and ID_CONTACTSLIST from environment variables and
        uses them to create an instance of the Mailjet `Client` for authenticated
        API interactions.

        Attributes:
        - self.auth (tuple[str, str]): A tuple containing the public and private API keys obtained from the environment variables 'MJ_APIKEY_PUBLIC' and 'MJ_APIKEY_PRIVATE' respectively.
        - self.client (Client): An instance of the Mailjet Client class, initialized with the provided authentication credentials.
        - self.id_contactslist (str): A string of the contacts list ID from https://app.mailjet.com/contacts
        """
        self.auth: tuple[str, str] = (
            os.environ["MJ_APIKEY_PUBLIC"],
            os.environ["MJ_APIKEY_PRIVATE"],
        )
        self.client: Client = Client(auth=self.auth)
        self.id_contactslist: str = os.environ["ID_CONTACTSLIST"]

    def test_01_upload_the_csv(self) -> None:
        """Test uploading a csv file.

        POST https://api.mailjet.com/v3/DATA/contactslist
        /$ID_CONTACTLIST/CSVData/text:plain
        """
        result = self.client.contactslist_csvdata.create(
            id=self.id_contactslist,
            data=Path("tests/doc_tests/files/data.csv").read_text(encoding="utf-8"),
        )
        self.assertEqual(result.status_code, 200)

        self.set_shared("data_id", result.json().get("ID"))
        data_id = self.get_shared("data_id")
        self.assertIsNotNone(data_id)

    def test_02_import_csv_content_to_a_list(self) -> None:
        """Test importing a csv content to a list.

        POST https://api.mailjet.com/v3/REST/csvimport
        """
        data_id = self.get_shared("data_id")
        self.assertIsNotNone(data_id)
        data = {
            "Method": "addnoforce",
            "ContactsListID": self.id_contactslist,
            "DataID": data_id,
        }
        result = self.client.csvimport.create(data=data)
        self.assertEqual(result.status_code, 201)
        self.assertIn("ID", result.json()["Data"][0])

        self.set_shared("id_value", result.json()["Data"][0]["ID"])

    def test_03_monitor_the_import_progress(self) -> None:
        """Test getting a csv content import.

        GET https://api.mailjet.com/v3/REST/csvimport/$importjob_ID
        """
        result = self.client.csvimport.get(id=self.get_shared("id_value"))
        self.assertEqual(result.status_code, 200)
        self.assertIn("ID", result.json()["Data"][0])
        self.assertEqual(0, result.json()["Data"][0]["Errcount"])


if __name__ == "__main__":
    unittest.main()



import json
import os

from mailjet_rest import Client


mailjet30 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
)

mailjet31 = Client(
    auth=(os.environ["MJ_APIKEY_PUBLIC"], os.environ["MJ_APIKEY_PRIVATE"]),
    version="v3.1",
)


def create_your_segment():
    """POST https://api.mailjet.com/v3/REST/contactfilter"""
    data = {
        "Description": "Will send only to contacts under 35 years of age.",
        "Expression": "(age<35)",
        "Name": "Customers under 35",
    }
    return mailjet30.contactfilter.create(data=data)


def create_a_campaign_with_a_segmentation_filter():
    """POST https://api.mailjet.com/v3/REST/newsletter"""
    data = {
        "Title": "Mailjet greets every contact over 40",
        "Locale": "en_US",
        "Sender": "MisterMailjet",
        "SenderEmail": "Mister@mailjet.com",
        "Subject": "Greetings from Mailjet",
        "ContactsListID": "$ID_CONTACTLIST",
        "SegmentationID": "$ID_CONTACT_FILTER",
    }
    return mailjet30.newsletter.create(data=data)


if __name__ == "__main__":
    result = create_your_segment()
    print(result.status_code)
    try:
        print(json.dumps(result.json(), indent=4))
    except json.decoder.JSONDecodeError:
        print(result.text)
